using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using Object = UnityEngine.Object;

namespace CheesyFX
{
    public class FlexRegionSupportTracker
    {
        private Transform origin;
        private FlexRegion flexRegion;
        private List<FlexCollisionTrigger> triggers;
        public HashSet<Rigidbody> stuffSupporting = new HashSet<Rigidbody>();
        private bool _supported;
        public bool colliding;
        public Rigidbody exclude;

        public Func<float> GetDownAngle;
        public bool supported
        {
            get { return _supported; }
            set
            {
                if (_supported == value) return;
                _supported = flexRegion.supported = value;
            }
        }
        private float onStayTimeout;

        public FlexRegionSupportTracker(FlexRegion flexRegion, Rigidbody rigidbody, List<FlexCollisionTrigger> triggers, Rigidbody exclude = null)
        {
            origin = rigidbody.transform;
            this.flexRegion = flexRegion;
            this.triggers = triggers;
            if (exclude != null) this.exclude = exclude;
            foreach (var trigger in triggers)
            {
                trigger.tracker = this;
            }

            if (flexRegion.id == 0)
            {
                JSONStorableBool heelColliding;
                Transform heelCollider;
                if (rigidbody.name.StartsWith("l"))
                {
                    heelColliding = FlexIt.lheelColliding;
                    heelCollider = FlexIt.lHeelCollider;
                }
                else
                {
                    heelColliding = FlexIt.lheelColliding;
                    heelCollider = FlexIt.lHeelCollider;
                }
                GetDownAngle = () =>
                {
                    if (heelColliding.val)
                    {
                        return heelCollider.up.y;
                    }
                    return origin.forward.y;
                };
            }
            // if (flexRegion.id == 0) origin.forward.y.Print();
            else GetDownAngle = () =>
            {
                // origin.Print();
                return -origin.up.y;
            };
        }
        
        public void RegisterCollision(Collision collision)
        {
            if (collision.rigidbody == exclude) return;
            // if(flexRegion.id == 1)
            // {
            //     var torque = Vector3.Cross(collision.contacts[0].point - origin.position,
            //         collision.impulse / Time.fixedDeltaTime);
            //     torque.Print();
            //     flexRegion.morphs[0][1].morphValue = -torque.x;
            // }
            if(!colliding) colliding = true;
            Vector3 contactVector = (collision.contacts[0].point - origin.position).normalized;
            float rbDown = GetDownAngle();
            // collision.collider.transform.Print();
            // $"{contactVector.y} {rbDown}".Print();
            var supporting = contactVector.y < rbDown;
            if(supporting)
            {
                
                stuffSupporting.Add(collision.rigidbody);
                if(!supported)
                {
                    supported = true;
                    OnStayRoutine().Start();
                }
            }
        }
        
        public void DeregisterCollision(Collision collision){
            stuffSupporting.Remove(collision.rigidbody);
            if(stuffSupporting.Count == 0)
            {
                colliding = false;
                onStayTimeout = .05f;
            }
        }
        
        public IEnumerator OnStayRoutine(){
            while(stuffSupporting.Count > 0 || onStayTimeout > 0f)
            {
                if(stuffSupporting.Count == 0) onStayTimeout -= Time.deltaTime;
                yield return new WaitForSeconds(Time.fixedDeltaTime);
            }
            supported = false;
            // supported.Print();
        }

        public void Destroy()
        {
            for (int i = 0; i < triggers.Count; i++)
            {
                Object.Destroy(triggers[i]);
            }
        }
        
        public void SetActive(bool val)
        {
            for (int i = 0; i < triggers.Count; i++)
            {
                triggers[i].enabled = val;
            }
        }
    }
}